Option Strict Off

Imports System.Runtime.InteropServices
Imports System.Text
Imports System.io

''' <summary>
''' Classe appelant la compression/dcompression IO interne de NT.
''' </summary>
''' <remarks></remarks>
Public Class NTInternalCompression

    ''' <summary>Methodes de compression pouvant etre utilises.</summary>
    Enum COMPRESS_METHOD
        _DEFAULT = 1
        _LZNT1 = 2
    End Enum


    ''' <summary>Compresse un fichier.</summary>
    ''' <param name="path">Chemin du fichier.</param>
    ''' <param name="method">Mthode de compression.</param>
    ''' <returns>Retourne le numro d'erreur.
    ''' 0 = Pas d'erreur, la compression a russi.
    ''' -2 = Le volume dans lequel se trouve le fichier ne supporte pas la compression de fichiers.</returns>
    Shared Function CompressFile(ByVal path As String, Optional ByVal method As COMPRESS_METHOD = COMPRESS_METHOD._LZNT1) As Integer

        If CompressionIsSupported(path.Substring(0, 3)) Then

            Dim hPath As IntPtr = NativeMethods.CreateFile("\\.\" & path, NativeMethods.GENERIC_ALL, NativeMethods.FILE_SHARE_NONE, IntPtr.Zero, NativeMethods.OPEN_EXISTING, 0, IntPtr.Zero)

            Dim iBytesReturned As Integer = 0

            NativeMethods.SetIoCompression(hPath, NativeMethods.FSCTL_SET_COMPRESSION, method, Marshal.SizeOf(GetType(Short)), IntPtr.Zero, 0, iBytesReturned, IntPtr.Zero)

            NativeMethods.CloseHandle(hPath)

        Else
            Return -2
        End If

        Return Marshal.GetLastWin32Error

    End Function

    ''' <summary>Dcompresse un fichier.</summary>
    ''' <param name="path">Chemin  dcompresser.</param>
    ''' <returns>Retourne le numro d'erreur. Si 0 pas d'erreur.
    ''' -1 = Le fichier n'est pas compress donc pas de dcompression possible.
    ''' -2 = Le volume dans lequel se trouve le fichier ne supporte pas la compression de fichiers.</returns>
    Shared Function UnCompressFile(ByVal path As String) As Integer

        If CompressionIsSupported(path.Substring(0, 3)) Then

            If FileCompressionState(path) Then

                Dim hPath As IntPtr = NativeMethods.CreateFile("\\.\" & path, NativeMethods.GENERIC_ALL, NativeMethods.FILE_SHARE_NONE, IntPtr.Zero, NativeMethods.OPEN_EXISTING, 0, IntPtr.Zero)

                Dim iBytesReturned As Integer = 0

                NativeMethods.SetIoCompression(hPath, NativeMethods.FSCTL_SET_COMPRESSION, NativeMethods.COMPRESSION_FORMAT_NONE, Marshal.SizeOf(GetType(Short)), IntPtr.Zero, 0, iBytesReturned, IntPtr.Zero)

                NativeMethods.CloseHandle(hPath)

            Else
                Return -1
            End If

        Else
            Return -2
        End If

        Return Marshal.GetLastWin32Error

    End Function

    ''' <summary>Indique si le fichier est compress.</summary>
    ''' <param name="path">Chemin du fichier.</param>
    ''' <returns>True si compress; False non compress.</returns>
    Shared Function FileCompressionState(ByVal path As String) As Boolean

        If CompressionIsSupported(path.Substring(0, 3)) Then

            Dim hPath As IntPtr = NativeMethods.CreateFile("\\.\" & path, NativeMethods.GENERIC_ALL, NativeMethods.FILE_SHARE_NONE, IntPtr.Zero, NativeMethods.OPEN_EXISTING, 0, IntPtr.Zero)

            Dim iMethod As Short = -1
            Dim iBytesReturned As Integer = 0

            NativeMethods.GetIoCompression(hPath, NativeMethods.FSCTL_GET_COMPRESSION, 0, 0, iMethod, Marshal.SizeOf(GetType(Short)), iBytesReturned, IntPtr.Zero)

            NativeMethods.CloseHandle(hPath)

            If iMethod <> 0 Then Return True Else Return False

        Else
            Return False
        End If

    End Function

    ''' <summary>Indique si la compression de fichiers est support sur le lecteur.</summary>
    ''' <param name="volume">Lettre du lecteur.</param>
    ''' <returns>True si russi; False si chou.</returns>
    Shared Function CompressionIsSupported(ByVal volume As String) As Boolean

        Dim szVolBuff As New StringBuilder
        Dim szFSBuff As New StringBuilder
        Dim flags As Integer = 0

        UnsafeMethods.GetVolumeInformation(volume, szVolBuff, 128, Nothing, Nothing, flags, szFSBuff, 128)

        If flags And UnsafeMethods.FS_FILE_COMPRESSION Then Return True Else Return False

    End Function

    ''' <summary>Compresse un rpertoire.</summary>
    ''' <param name="path">Chemin du rpertoire.</param>
    ''' <param name="recursive">Inclut les sous-rpertoires.</param>
    ''' <param name="method">Mthode de compression.</param>
    Shared Sub CompressDirectory(ByVal path As String, ByVal recursive As Boolean, Optional ByVal method As COMPRESS_METHOD = COMPRESS_METHOD._LZNT1)

        If CompressionIsSupported(path.Substring(0, 3)) Then

            Dim hPath As IntPtr = NativeMethods.CreateFile("\\.\" & path, NativeMethods.GENERIC_ALL, NativeMethods.FILE_SHARE_NONE, IntPtr.Zero, NativeMethods.OPEN_EXISTING, NativeMethods.FILE_FLAG_BACKUP_SEMANTICS, IntPtr.Zero)

            Dim iBytesReturned As Integer = 0

            NativeMethods.SetIoCompression(hPath, NativeMethods.FSCTL_SET_COMPRESSION, method, Marshal.SizeOf(GetType(Short)), IntPtr.Zero, 0, iBytesReturned, IntPtr.Zero)

            NativeMethods.CloseHandle(hPath)


            'ajoute l'attribut de compression aux rpertoires
            For Each curdir As String In Directory.GetDirectories(path, "*", Convert.ToInt32(recursive))

                hPath = NativeMethods.CreateFile("\\.\" & curdir, NativeMethods.GENERIC_ALL, NativeMethods.FILE_SHARE_NONE, IntPtr.Zero, NativeMethods.OPEN_EXISTING, NativeMethods.FILE_FLAG_BACKUP_SEMANTICS, IntPtr.Zero)

                NativeMethods.SetIoCompression(hPath, NativeMethods.FSCTL_SET_COMPRESSION, method, Marshal.SizeOf(GetType(Short)), IntPtr.Zero, 0, iBytesReturned, IntPtr.Zero)

                NativeMethods.CloseHandle(hPath)

            Next

            'Compresse tout les fichiers trouvs dans les rpertoires
            Dim files As String() = Directory.GetFiles(path, "*", Convert.ToInt32(recursive))
            If files.Length > 0 Then

                For i As Integer = 0 To files.Length - 1

                    hPath = NativeMethods.CreateFile("\\.\" & files(i), NativeMethods.GENERIC_ALL, NativeMethods.FILE_SHARE_NONE, IntPtr.Zero, NativeMethods.OPEN_EXISTING, 0, IntPtr.Zero)

                    NativeMethods.SetIoCompression(hPath, NativeMethods.FSCTL_SET_COMPRESSION, method, Marshal.SizeOf(GetType(Short)), IntPtr.Zero, 0, iBytesReturned, IntPtr.Zero)

                    NativeMethods.CloseHandle(hPath)

                Next

            End If

        Else

#If DEBUG Then
            Debug.WriteLine("Compression non support sur ce lecteur.")
#End If

        End If

    End Sub

End Class
